"use strict";
var dependency_1 = require("./dependency");
require("source-map-support/register");
var async = require("async");
var fs = require("fs");
var pathModule = require("path");
var colors = require("colors/safe");
var Leaf = require("leaf-ts");
var sharedCallbacks_1 = require("./sharedCallbacks");
var useColors = colors;
var ServiceContext = (function () {
    function ServiceContext() {
        this.services = {};
        this.dependencyManager = new dependency_1.DependencyManager;
        this.supportedLoadExtnames = [".ts", ".js"];
        this.maxInitializingTimeout = 1000 * 5;
        this.isSetup = false;
        this.finishSetup = sharedCallbacks_1.SharedCallbacks.create();
    }
    ServiceContext.prototype.register = function (s) {
        this.dependencyManager.add(s.name, s.dependencies || null);
        this.services[s.name] = s;
    };
    ServiceContext.prototype.loadServices = function (path) {
        var _this = this;
        path = pathModule.resolve(path);
        var files = fs.readdirSync(path);
        files = files.filter(function (item) {
            var ext = pathModule.extname(item);
            if (_this.supportedLoadExtnames.indexOf(ext) < 0) {
                return false;
            }
            return true;
        });
        files = files.map(function (item) {
            var ext = pathModule.extname(item);
            var base = pathModule.basename(item, ext);
            return pathModule.join(path, base);
        });
        var toLoads = [];
        for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
            var file = files_1[_i];
            if (toLoads.indexOf(file) >= 0)
                continue;
            toLoads.push(file);
        }
        for (var _a = 0, toLoads_1 = toLoads; _a < toLoads_1.length; _a++) {
            var file = toLoads_1[_a];
            this.loadService(file);
        }
    };
    ServiceContext.prototype.loadService = function (path) {
        var result = require(path) || {};
        for (var name_1 in result) {
            var obj = result[name_1];
            if (this.isServiceConstructor(obj)) {
                var Cons = obj;
                var s = new Cons();
                if (!s.name) {
                    console.log(useColors.cyan("Detect service without name. Use constructor name:" + Cons.name));
                    s.name = Cons.name;
                }
                console.log("Load service " + useColors.green(Leaf.Util.capitalize(s.name)));
                this.register(s);
            }
        }
    };
    ServiceContext.prototype.isServiceConstructor = function (obj) {
        return typeof obj === "function" && obj.prototype["thisObjectIsRootServiceConstructor"];
    };
    ServiceContext.prototype.setup = function (finish) {
        var _this = this;
        if (finish === void 0) { finish = function () { return null; }; }
        this.finishSetup.push(finish);
        if (this.isSetup) {
            return;
        }
        this.isSetup = true;
        var initialSequence = this.dependencyManager.resolve();
        var timer = null;
        async.eachSeries(initialSequence, function (name, done) {
            var service = _this.services[name];
            if (service.isInitialized) {
                done();
                return;
            }
            service.services = _this.services;
            if (service.initialize) {
                if (service.initialize.length == 1) {
                    timer = setTimeout(function () {
                        console.error("Service " + name + " initializing timeout, exit program");
                        process.exit(1);
                    }, _this.maxInitializingTimeout);
                    service.initialize(function (err) {
                        clearTimeout(timer);
                        if (err) {
                            console.error(useColors.red("Fail to initialize service " + name + "."));
                            done(err);
                            return;
                        }
                        service.isInitialized = true;
                        _this.logInitialized(name);
                        done();
                    });
                }
                else {
                    service.initialize();
                    service.isInitialized = true;
                    _this.logInitialized(name);
                    done();
                }
            }
            else {
                // this service don't need initialization
                service.isInitialized = true;
                _this.logInitialized(name);
                done();
            }
        }, function (err) {
            _this.isSetup = false;
            if (err) {
                _this.finishSetup(err);
                return;
            }
            console.log(useColors.bgGreen("All service initialized"));
            _this.finishSetup();
        });
    };
    ServiceContext.prototype.logInitialized = function (name) {
        console.log("Service " + useColors.green(name) + " Initialized");
    };
    return ServiceContext;
}());
exports.ServiceContext = ServiceContext;
var Service = (function () {
    function Service() {
        this.services = null;
        this.dependencies = [];
        this.isInitialized = false;
    }
    Service.prototype.initialize = function (done) {
        done();
    };
    return Service;
}());
exports.Service = Service;
Service.prototype["thisObjectIsRootServiceConstructor"] = true;
//# sourceMappingURL=service.js.map